@@ -3,7 +3,7 @@
  * socket.c
  *	  Microsoft Windows Win32 Socket Functions
  *
- * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *	  src/backend/port/win32/socket.c
@@ -27,10 +27,7 @@
  */
 int			pgwin32_noblock = 0;
 
-/* Undef the macros defined in win32.h, so we can access system functions */
 #undef socket
-#undef bind
-#undef listen
 #undef accept
 #undef connect
 #undef select
@@ -44,37 +41,22 @@ int			pgwin32_noblock = 0;
 
 /*
  * Convert the last socket error code into errno
- *
- * Note: where there is a direct correspondence between a WSAxxx error code
- * and a Berkeley error symbol, this mapping is actually a no-op, because
- * in win32.h we redefine the network-related Berkeley error symbols to have
- * the values of their WSAxxx counterparts.  The point of the switch is
- * mostly to translate near-miss error codes into something that's sensible
- * in the Berkeley universe.
  */
 static void
 TranslateSocketError(void)
 {
 	switch (WSAGetLastError())
 	{
-		case WSAEINVAL:
 		case WSANOTINITIALISED:
+		case WSAENETDOWN:
+		case WSAEINPROGRESS:
+		case WSAEINVAL:
+		case WSAESOCKTNOSUPPORT:
+		case WSAEFAULT:
 		case WSAEINVALIDPROVIDER:
 		case WSAEINVALIDPROCTABLE:
-		case WSAEDESTADDRREQ:
-			errno = EINVAL;
-			break;
-		case WSAEINPROGRESS:
-			errno = EINPROGRESS;
-			break;
-		case WSAEFAULT:
-			errno = EFAULT;
-			break;
-		case WSAEISCONN:
-			errno = EISCONN;
-			break;
 		case WSAEMSGSIZE:
-			errno = EMSGSIZE;
+			errno = EINVAL;
 			break;
 		case WSAEAFNOSUPPORT:
 			errno = EAFNOSUPPORT;
@@ -87,23 +69,16 @@ TranslateSocketError(void)
 			break;
 		case WSAEPROTONOSUPPORT:
 		case WSAEPROTOTYPE:
-		case WSAESOCKTNOSUPPORT:
 			errno = EPROTONOSUPPORT;
 			break;
-		case WSAECONNABORTED:
-			errno = ECONNABORTED;
-			break;
 		case WSAECONNREFUSED:
 			errno = ECONNREFUSED;
 			break;
-		case WSAECONNRESET:
-			errno = ECONNRESET;
-			break;
 		case WSAEINTR:
 			errno = EINTR;
 			break;
 		case WSAENOTSOCK:
-			errno = ENOTSOCK;
+			errno = EBADFD;
 			break;
 		case WSAEOPNOTSUPP:
 			errno = EOPNOTSUPP;
@@ -114,24 +89,13 @@ TranslateSocketError(void)
 		case WSAEACCES:
 			errno = EACCES;
 			break;
-		case WSAEADDRINUSE:
-			errno = EADDRINUSE;
-			break;
-		case WSAEADDRNOTAVAIL:
-			errno = EADDRNOTAVAIL;
-			break;
-		case WSAEHOSTUNREACH:
-		case WSAEHOSTDOWN:
-		case WSAHOST_NOT_FOUND:
-		case WSAENETDOWN:
-		case WSAENETUNREACH:
-		case WSAENETRESET:
-			errno = EHOSTUNREACH;
-			break;
 		case WSAENOTCONN:
+		case WSAENETRESET:
+		case WSAECONNRESET:
 		case WSAESHUTDOWN:
+		case WSAECONNABORTED:
 		case WSAEDISCON:
-			errno = ENOTCONN;
+			errno = ECONNREFUSED;		/* ENOTCONN? */
 			break;
 		default:
 			ereport(NOTICE,
@@ -213,7 +177,7 @@ pgwin32_waitforsinglesocket(SOCKET s, int what, int timeout)
 	 * Just a workaround of unknown locking problem with writing in UDP socket
 	 * under high load: Client's pgsql backend sleeps infinitely in
 	 * WaitForMultipleObjectsEx, pgstat process sleeps in pgwin32_select().
-	 * So, we will wait with small timeout(0.1 sec) and if sockect is still
+	 * So, we will wait with small timeout(0.1 sec) and if socket is still
 	 * blocked, try WSASend (see comments in pgwin32_select) and wait again.
 	 */
 	if ((what & FD_WRITE) && isUDP)
@@ -297,27 +261,6 @@ pgwin32_socket(int af, int type, int protocol)
 	return s;
 }
 
-int
-pgwin32_bind(SOCKET s, struct sockaddr * addr, int addrlen)
-{
-	int			res;
-
-	res = bind(s, addr, addrlen);
-	if (res < 0)
-		TranslateSocketError();
-	return res;
-}
-
-int
-pgwin32_listen(SOCKET s, int backlog)
-{
-	int			res;
-
-	res = listen(s, backlog);
-	if (res < 0)
-		TranslateSocketError();
-	return res;
-}
 
 SOCKET
 pgwin32_accept(SOCKET s, struct sockaddr * addr, int *addrlen)
@@ -675,7 +618,8 @@ pgwin32_select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, c
 		return 0;
 	}
 
-	if (r == WAIT_OBJECT_0 + numevents)
+	/* Signal-like events. */
+	if (r == WAIT_OBJECT_0 + numevents || r == WAIT_IO_COMPLETION)
 	{
 		pgwin32_dispatch_queued_signals();
 		errno = EINTR;
@@ -714,9 +658,7 @@ pgwin32_socket_strerror(int err)
 	}
 
 	ZeroMemory(&wserrbuf, sizeof(wserrbuf));
-	if (FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS |
-					  FORMAT_MESSAGE_FROM_SYSTEM |
-					  FORMAT_MESSAGE_FROM_HMODULE,
+	if (FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_FROM_HMODULE,
 					  handleDLL,
 					  err,
 					  MAKELANGID(LANG_ENGLISH, SUBLANG_DEFAULT),
