@@ -3,7 +3,7 @@
  * dict_snowball.c
  *		Snowball dictionary
  *
- * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *	  src/backend/snowball/dict_snowball.c
@@ -58,14 +58,11 @@
 #include "snowball/libstemmer/stem_UTF_8_swedish.h"
 #include "snowball/libstemmer/stem_UTF_8_turkish.h"
 
-
 PG_MODULE_MAGIC;
 
 PG_FUNCTION_INFO_V1(dsnowball_init);
-Datum		dsnowball_init(PG_FUNCTION_ARGS);
 
 PG_FUNCTION_INFO_V1(dsnowball_lexize);
-Datum		dsnowball_lexize(PG_FUNCTION_ARGS);
 
 /* List of supported modules */
 typedef struct stemmer_module
