@@ -621,7 +621,7 @@ cash_mul_flt8(PG_FUNCTION_ARGS)
 	float8		f = PG_GETARG_FLOAT8(1);
 	Cash		result;
 
-	result = rint(c * f);
+	result = c * f;
 	PG_RETURN_CASH(result);
 }
 
@@ -636,7 +636,7 @@ flt8_mul_cash(PG_FUNCTION_ARGS)
 	Cash		c = PG_GETARG_CASH(1);
 	Cash		result;
 
-	result = rint(f * c);
+	result = f * c;
 	PG_RETURN_CASH(result);
 }
 
@@ -671,7 +671,7 @@ cash_mul_flt4(PG_FUNCTION_ARGS)
 	float4		f = PG_GETARG_FLOAT4(1);
 	Cash		result;
 
-	result = rint(c * (float8) f);
+	result = c * f;
 	PG_RETURN_CASH(result);
 }
 
@@ -686,7 +686,7 @@ flt4_mul_cash(PG_FUNCTION_ARGS)
 	Cash		c = PG_GETARG_CASH(1);
 	Cash		result;
 
-	result = rint((float8) f * c);
+	result = f * c;
 	PG_RETURN_CASH(result);
 }
 
@@ -707,7 +707,7 @@ cash_div_flt4(PG_FUNCTION_ARGS)
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	result = rint(c / (float8) f);
+	result = rint(c / f);
 	PG_RETURN_CASH(result);
 }
 
@@ -756,7 +756,7 @@ cash_div_int8(PG_FUNCTION_ARGS)
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	result = c / i;
+	result = rint(c / i);
 
 	PG_RETURN_CASH(result);
 }
@@ -808,7 +808,7 @@ cash_div_int4(PG_FUNCTION_ARGS)
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	result = c / i;
+	result = rint(c / i);
 
 	PG_RETURN_CASH(result);
 }
@@ -858,7 +858,7 @@ cash_div_int2(PG_FUNCTION_ARGS)
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	result = c / s;
+	result = rint(c / s);
 	PG_RETURN_CASH(result);
 }
 
@@ -893,7 +893,7 @@ cashsmaller(PG_FUNCTION_ARGS)
 }
 
 /* cash_words()
- * This converts a int4 as well but to a representation using words
+ * This converts an int4 as well but to a representation using words
  * Obviously way North American centric - sorry
  */
 Datum
@@ -987,8 +987,13 @@ Datum
 cash_numeric(PG_FUNCTION_ARGS)
 {
 	Cash		money = PG_GETARG_CASH(0);
-	Datum		result;
+	Numeric		result;
 	int			fpoint;
+	int64		scale;
+	int			i;
+	Datum		amount;
+	Datum		numeric_scale;
+	Datum		quotient;
 	struct lconv *lconvert = PGLC_localeconv();
 
 	/* see comments about frac_digits in cash_in() */
@@ -996,45 +1001,22 @@ cash_numeric(PG_FUNCTION_ARGS)
 	if (fpoint < 0 || fpoint > 10)
 		fpoint = 2;
 
-	/* convert the integral money value to numeric */
-	result = DirectFunctionCall1(int8_numeric, Int64GetDatum(money));
+	/* compute required scale factor */
+	scale = 1;
+	for (i = 0; i < fpoint; i++)
+		scale *= 10;
 
-	/* scale appropriately, if needed */
-	if (fpoint > 0)
-	{
-		int64		scale;
-		int			i;
-		Datum		numeric_scale;
-		Datum		quotient;
+	/* form the result as money / scale */
+	amount = DirectFunctionCall1(int8_numeric, Int64GetDatum(money));
+	numeric_scale = DirectFunctionCall1(int8_numeric, Int64GetDatum(scale));
+	quotient = DirectFunctionCall2(numeric_div, amount, numeric_scale);
 
-		/* compute required scale factor */
-		scale = 1;
-		for (i = 0; i < fpoint; i++)
-			scale *= 10;
-		numeric_scale = DirectFunctionCall1(int8_numeric,
-											Int64GetDatum(scale));
+	/* forcibly round to exactly the intended number of digits */
+	result = DatumGetNumeric(DirectFunctionCall2(numeric_round,
+												 quotient,
+												 Int32GetDatum(fpoint)));
 
-		/*
-		 * Given integral inputs approaching INT64_MAX, select_div_scale()
-		 * might choose a result scale of zero, causing loss of fractional
-		 * digits in the quotient.  We can ensure an exact result by setting
-		 * the dscale of either input to be at least as large as the desired
-		 * result scale.  numeric_round() will do that for us.
-		 */
-		numeric_scale = DirectFunctionCall2(numeric_round,
-											numeric_scale,
-											Int32GetDatum(fpoint));
-
-		/* Now we can safely divide ... */
-		quotient = DirectFunctionCall2(numeric_div, result, numeric_scale);
-
-		/* ... and forcibly round to exactly the intended number of digits */
-		result = DirectFunctionCall2(numeric_round,
-									 quotient,
-									 Int32GetDatum(fpoint));
-	}
-
-	PG_RETURN_DATUM(result);
+	PG_RETURN_NUMERIC(result);
 }
 
 /* numeric_cash()
