@@ -30,7 +30,8 @@ static const char rcsid[] = "Id: inet_net_pton.c,v 1.4.2.3 2004/03/17 00:40:11 m
 #include <assert.h>
 #include <ctype.h>
 
-#include "utils/builtins.h"
+#include "utils/builtins.h" /* pgrminclude ignore */	/* needed on some
+														 * platforms */
 #include "utils/inet.h"
 
 
@@ -495,7 +496,7 @@ inet_cidr_pton_ipv6(const char *src, u_char *dst, size_t size)
 			else if (*src == '\0')
 				goto enoent;
 			if (tp + NS_INT16SZ > endp)
-				goto enoent;
+				return (0);
 			*tp++ = (u_char) (val >> 8) & 0xff;
 			*tp++ = (u_char) val & 0xff;
 			saw_xdigit = 0;
