@@ -6,7 +6,7 @@
  * Estimates are based on histograms of lower and upper bounds, and the
  * fraction of empty ranges.
  *
- * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -20,7 +20,6 @@
 #include "access/htup_details.h"
 #include "catalog/pg_operator.h"
 #include "catalog/pg_statistic.h"
-#include "catalog/pg_type.h"
 #include "utils/builtins.h"
 #include "utils/lsyscache.h"
 #include "utils/rangetypes.h"
@@ -247,9 +246,8 @@ calc_rangesel(TypeCacheEntry *typcache, VariableStatData *vardata,
 
 		/* Try to get fraction of empty ranges */
 		if (get_attstatsslot(vardata->statsTuple,
-							 FLOAT8OID, -1,
-							 STATISTIC_KIND_RANGE_LENGTH_HISTOGRAM,
-							 InvalidOid,
+							 vardata->atttype, vardata->atttypmod,
+						   STATISTIC_KIND_RANGE_LENGTH_HISTOGRAM, InvalidOid,
 							 NULL,
 							 NULL, NULL,
 							 &numbers, &nnumbers))
@@ -257,7 +255,6 @@ calc_rangesel(TypeCacheEntry *typcache, VariableStatData *vardata,
 			if (nnumbers != 1)
 				elog(ERROR, "invalid empty fraction statistic");		/* shouldn't happen */
 			empty_frac = numbers[0];
-			free_attstatsslot(FLOAT8OID, NULL, 0, numbers, nnumbers);
 		}
 		else
 		{
@@ -376,8 +373,8 @@ calc_hist_selectivity(TypeCacheEntry *typcache, VariableStatData *vardata,
 {
 	Datum	   *hist_values;
 	int			nhist;
-	Datum	   *length_hist_values = NULL;
-	int			length_nhist = 0;
+	Datum	   *length_hist_values;
+	int			length_nhist;
 	RangeBound *hist_lower;
 	RangeBound *hist_upper;
 	int			i;
@@ -386,15 +383,6 @@ calc_hist_selectivity(TypeCacheEntry *typcache, VariableStatData *vardata,
 	bool		empty;
 	double		hist_selec;
 
-	/* Can't use the histogram with insecure range support functions */
-	if (!statistic_proc_security_check(vardata,
-									   typcache->rng_cmp_proc_finfo.fn_oid))
-		return -1;
-	if (OidIsValid(typcache->rng_subdiff_finfo.fn_oid) &&
-		!statistic_proc_security_check(vardata,
-									   typcache->rng_subdiff_finfo.fn_oid))
-		return -1;
-
 	/* Try to get histogram of ranges */
 	if (!(HeapTupleIsValid(vardata->statsTuple) &&
 		  get_attstatsslot(vardata->statsTuple,
@@ -426,25 +414,17 @@ calc_hist_selectivity(TypeCacheEntry *typcache, VariableStatData *vardata,
 	{
 		if (!(HeapTupleIsValid(vardata->statsTuple) &&
 			  get_attstatsslot(vardata->statsTuple,
-							   FLOAT8OID, -1,
+							   vardata->atttype, vardata->atttypmod,
 							   STATISTIC_KIND_RANGE_LENGTH_HISTOGRAM,
 							   InvalidOid,
 							   NULL,
 							   &length_hist_values, &length_nhist,
 							   NULL, NULL)))
-		{
-			free_attstatsslot(vardata->atttype, hist_values, nhist, NULL, 0);
 			return -1.0;
-		}
 
 		/* check that it's a histogram, not just a dummy entry */
 		if (length_nhist < 2)
-		{
-			free_attstatsslot(FLOAT8OID,
-							  length_hist_values, length_nhist, NULL, 0);
-			free_attstatsslot(vardata->atttype, hist_values, nhist, NULL, 0);
 			return -1.0;
-		}
 	}
 
 	/* Extract the bounds of the constant value. */
@@ -553,7 +533,7 @@ calc_hist_selectivity(TypeCacheEntry *typcache, VariableStatData *vardata,
 			{
 				/*
 				 * Lower bound no longer matters. Just estimate the fraction
-				 * with an upper bound <= const upper bound
+				 * with an upper bound <= const uppert bound
 				 */
 				hist_selec =
 					calc_hist_selectivity_scalar(typcache, &const_upper,
@@ -580,10 +560,6 @@ calc_hist_selectivity(TypeCacheEntry *typcache, VariableStatData *vardata,
 			break;
 	}
 
-	free_attstatsslot(FLOAT8OID,
-					  length_hist_values, length_nhist, NULL, 0);
-	free_attstatsslot(vardata->atttype, hist_values, nhist, NULL, 0);
-
 	return hist_selec;
 }
 
@@ -620,8 +596,8 @@ calc_hist_selectivity_scalar(TypeCacheEntry *typcache, RangeBound *constbound,
  * range bounds in array are greater or equal(greater) than given range bound,
  * return -1. When "equal" flag is set conditions in brackets are used.
  *
- * This function is used in scalar operators selectivity estimation. Another
- * goal of this function is to found an histogram bin where to stop
+ * This function is used in scalar operator selectivity estimation. Another
+ * goal of this function is to find a histogram bin where to stop
  * interpolation of portion of bounds which are less or equal to given bound.
  */
 static int
