@@ -2,7 +2,7 @@
  *
  *	  ASCII <--> UTF8
  *
- * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -20,9 +20,6 @@ PG_MODULE_MAGIC;
 PG_FUNCTION_INFO_V1(ascii_to_utf8);
 PG_FUNCTION_INFO_V1(utf8_to_ascii);
 
-extern Datum ascii_to_utf8(PG_FUNCTION_ARGS);
-extern Datum utf8_to_ascii(PG_FUNCTION_ARGS);
-
 /* ----------
  * conv_proc(
  *		INTEGER,	-- source encoding id
