@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2009, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2015, PostgreSQL Global Development Group
  *
- * $PostgreSQL: pgsql/src/bin/psql/stringutils.h,v 1.27 2009/01/01 17:23:55 momjian Exp $
+ * src/bin/psql/stringutils.h
  */
 #ifndef STRINGUTILS_H
 #define STRINGUTILS_H
@@ -19,4 +19,9 @@ extern char *strtokx(const char *s,
 		bool del_quotes,
 		int encoding);
 
+extern void strip_quotes(char *source, char quote, char escape, int encoding);
+
+extern char *quote_if_needed(const char *source, const char *entails_quote,
+				char quote, char escape, int encoding);
+
 #endif   /* STRINGUTILS_H */
