@@ -4,7 +4,7 @@
  *	  Declarations for XML data type support.
  *
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/utils/xml.h
@@ -21,6 +21,31 @@
 
 typedef struct varlena xmltype;
 
+typedef enum
+{
+	XML_STANDALONE_YES,
+	XML_STANDALONE_NO,
+	XML_STANDALONE_NO_VALUE,
+	XML_STANDALONE_OMITTED
+}	XmlStandaloneType;
+
+typedef enum
+{
+	XMLBINARY_BASE64,
+	XMLBINARY_HEX
+}	XmlBinaryType;
+
+typedef enum
+{
+	PG_XML_STRICTNESS_LEGACY,	/* ignore errors unless function result
+								 * indicates error condition */
+	PG_XML_STRICTNESS_WELLFORMED,		/* ignore non-parser messages */
+	PG_XML_STRICTNESS_ALL		/* report all notices/warnings/errors */
+} PgXmlStrictness;
+
+/* struct PgXmlErrorContext is private to xml.c */
+typedef struct PgXmlErrorContext PgXmlErrorContext;
+
 #define DatumGetXmlP(X)		((xmltype *) PG_DETOAST_DATUM(X))
 #define XmlPGetDatum(X)		PointerGetDatum(X)
 
@@ -60,16 +85,13 @@ extern Datum database_to_xml(PG_FUNCTION_ARGS);
 extern Datum database_to_xmlschema(PG_FUNCTION_ARGS);
 extern Datum database_to_xml_and_xmlschema(PG_FUNCTION_ARGS);
 
-typedef enum
-{
-	XML_STANDALONE_YES,
-	XML_STANDALONE_NO,
-	XML_STANDALONE_NO_VALUE,
-	XML_STANDALONE_OMITTED
-}	XmlStandaloneType;
+extern void pg_xml_init_library(void);
+extern PgXmlErrorContext *pg_xml_init(PgXmlStrictness strictness);
+extern void pg_xml_done(PgXmlErrorContext *errcxt, bool isError);
+extern bool pg_xml_error_occurred(PgXmlErrorContext *errcxt);
+extern void xml_ereport(PgXmlErrorContext *errcxt, int level, int sqlcode,
+			const char *msg);
 
-extern void pg_xml_init(void);
-extern void xml_ereport(int level, int sqlcode, const char *msg);
 extern xmltype *xmlconcat(List *args);
 extern xmltype *xmlelement(XmlExprState *xmlExpr, ExprContext *econtext);
 extern xmltype *xmlparse(text *data, XmlOptionType xmloption, bool preserve_whitespace);
@@ -83,12 +105,6 @@ extern char *map_sql_identifier_to_xml_name(char *ident, bool fully_escaped, boo
 extern char *map_xml_name_to_sql_identifier(char *name);
 extern char *map_sql_value_to_xml_value(Datum value, Oid type, bool xml_escape_strings);
 
-typedef enum
-{
-	XMLBINARY_BASE64,
-	XMLBINARY_HEX
-}	XmlBinaryType;
-
 extern int	xmlbinary;			/* XmlBinaryType, but int for guc enum */
 
 extern int	xmloption;			/* XmlOptionType, but int for guc enum */
