@@ -3,7 +3,7 @@
  * fe-exec.c
  *	  functions related to sending a query down to the backend
  *
- * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -892,7 +892,8 @@ pqSaveMessageField(PGresult *res, char code, const char *value)
 
 	pfield = (PGMessageField *)
 		pqResultAlloc(res,
-					  sizeof(PGMessageField) + strlen(value),
+					  offsetof(PGMessageField, contents) +
+					  strlen(value) + 1,
 					  TRUE);
 	if (!pfield)
 		return;					/* out of memory? */
@@ -981,31 +982,14 @@ pqSaveParameterStatus(PGconn *conn, const char *name, const char *value)
 
 		cnt = sscanf(value, "%d.%d.%d", &vmaj, &vmin, &vrev);
 
-		if (cnt == 3)
+		if (cnt < 2)
+			conn->sversion = 0; /* unknown */
+		else
 		{
-			/* old style, e.g. 9.6.1 */
+			if (cnt == 2)
+				vrev = 0;
 			conn->sversion = (100 * vmaj + vmin) * 100 + vrev;
 		}
-		else if (cnt == 2)
-		{
-			if (vmaj >= 10)
-			{
-				/* new style, e.g. 10.1 */
-				conn->sversion = 100 * 100 * vmaj + vmin;
-			}
-			else
-			{
-				/* old style without minor version, e.g. 9.6devel */
-				conn->sversion = (100 * vmaj + vmin) * 100;
-			}
-		}
-		else if (cnt == 1)
-		{
-			/* new style without minor version, e.g. 10devel */
-			conn->sversion = 100 * 100 * vmaj;
-		}
-		else
-			conn->sversion = 0; /* unknown */
 	}
 }
 
@@ -1383,7 +1367,8 @@ PQsendQueryStart(PGconn *conn)
 	}
 
 	/* initialize async result-accumulation state */
-	pqClearAsyncResult(conn);
+	conn->result = NULL;
+	conn->next_result = NULL;
 
 	/* reset single-row processing mode */
 	conn->singleRowMode = false;
@@ -2924,9 +2909,9 @@ PQoidStatus(const PGresult *res)
 		return "";
 
 	len = strspn(res->cmdStatus + 7, "0123456789");
-	if (len > 23)
-		len = 23;
-	strncpy(buf, res->cmdStatus + 7, len);
+	if (len > sizeof(buf) - 1)
+		len = sizeof(buf) - 1;
+	memcpy(buf, res->cmdStatus + 7, len);
 	buf[len] = '\0';
 
 	return buf;
