@@ -69,6 +69,15 @@ DELETE FROM rw_view14 WHERE a=3; -- should be OK
 -- Partially updatable view
 INSERT INTO rw_view15 VALUES (3, 'ROW 3'); -- should fail
 INSERT INTO rw_view15 (a) VALUES (3); -- should be OK
+INSERT INTO rw_view15 (a) VALUES (3) ON CONFLICT DO NOTHING; -- succeeds
+SELECT * FROM rw_view15;
+INSERT INTO rw_view15 (a) VALUES (3) ON CONFLICT (a) DO NOTHING; -- succeeds
+SELECT * FROM rw_view15;
+INSERT INTO rw_view15 (a) VALUES (3) ON CONFLICT (a) DO UPDATE set a = excluded.a; -- succeeds
+SELECT * FROM rw_view15;
+INSERT INTO rw_view15 (a) VALUES (3) ON CONFLICT (a) DO UPDATE set upper = 'blarg'; -- fails
+SELECT * FROM rw_view15;
+SELECT * FROM rw_view15;
 ALTER VIEW rw_view15 ALTER COLUMN upper SET DEFAULT 'NOT SET';
 INSERT INTO rw_view15 (a) VALUES (4); -- should fail
 UPDATE rw_view15 SET upper='ROW 3' WHERE a=3; -- should fail
@@ -958,7 +967,7 @@ SELECT * FROM base_tbl;
 
 DROP TABLE base_tbl CASCADE;
 
--- security barrier view based on inheiritance set
+-- security barrier view based on inheritance set
 CREATE TABLE t1 (a int, b float, c text);
 CREATE INDEX t1_a_idx ON t1(a);
 INSERT INTO t1
@@ -1055,153 +1064,3 @@ DROP VIEW vx1;
 DROP TABLE tx1;
 DROP TABLE tx2;
 DROP TABLE tx3;
-
---
--- Test handling of vars from correlated subqueries in quals from outer
--- security barrier views, per bug #13988
---
-CREATE TABLE t1 (a int, b text, c int);
-INSERT INTO t1 VALUES (1, 'one', 10);
-
-CREATE TABLE t2 (cc int);
-INSERT INTO t2 VALUES (10), (20);
-
-CREATE VIEW v1 WITH (security_barrier = true) AS
-  SELECT * FROM t1 WHERE (a > 0)
-  WITH CHECK OPTION;
-
-CREATE VIEW v2 WITH (security_barrier = true) AS
-  SELECT * FROM v1 WHERE EXISTS (SELECT 1 FROM t2 WHERE t2.cc = v1.c)
-  WITH CHECK OPTION;
-
-INSERT INTO v2 VALUES (2, 'two', 20); -- ok
-INSERT INTO v2 VALUES (-2, 'minus two', 20); -- not allowed
-INSERT INTO v2 VALUES (3, 'three', 30); -- not allowed
-
-UPDATE v2 SET b = 'ONE' WHERE a = 1; -- ok
-UPDATE v2 SET a = -1 WHERE a = 1; -- not allowed
-UPDATE v2 SET c = 30 WHERE a = 1; -- not allowed
-
-DELETE FROM v2 WHERE a = 2; -- ok
-SELECT * FROM v2;
-
-DROP VIEW v2;
-DROP VIEW v1;
-DROP TABLE t2;
-DROP TABLE t1;
-
---
--- Test CREATE OR REPLACE VIEW turning a non-updatable view into an
--- auto-updatable view and adding check options in a single step
---
-CREATE TABLE t1 (a int, b text);
-CREATE VIEW v1 AS SELECT null::int AS a;
-CREATE OR REPLACE VIEW v1 AS SELECT * FROM t1 WHERE a > 0 WITH CHECK OPTION;
-
-INSERT INTO v1 VALUES (1, 'ok'); -- ok
-INSERT INTO v1 VALUES (-1, 'invalid'); -- should fail
-
-DROP VIEW v1;
-DROP TABLE t1;
-
--- Test single- and multi-row inserts with table and view defaults.
--- Table defaults should be used, unless overridden by view defaults.
-create table base_tab_def (a int, b text default 'Table default',
-                           c text default 'Table default', d text, e text);
-create view base_tab_def_view as select * from base_tab_def;
-alter view base_tab_def_view alter b set default 'View default';
-alter view base_tab_def_view alter d set default 'View default';
-insert into base_tab_def values (1);
-insert into base_tab_def values (2), (3);
-insert into base_tab_def values (4, default, default, default, default);
-insert into base_tab_def values (5, default, default, default, default),
-                                (6, default, default, default, default);
-insert into base_tab_def_view values (11);
-insert into base_tab_def_view values (12), (13);
-insert into base_tab_def_view values (14, default, default, default, default);
-insert into base_tab_def_view values (15, default, default, default, default),
-                                     (16, default, default, default, default);
-insert into base_tab_def_view values (17), (default);
-select * from base_tab_def order by a;
-
--- Adding an INSTEAD OF trigger should cause NULLs to be inserted instead of
--- table defaults, where there are no view defaults.
-create function base_tab_def_view_instrig_func() returns trigger
-as
-$$
-begin
-  insert into base_tab_def values (new.a, new.b, new.c, new.d, new.e);
-  return new;
-end;
-$$
-language plpgsql;
-create trigger base_tab_def_view_instrig instead of insert on base_tab_def_view
-  for each row execute procedure base_tab_def_view_instrig_func();
-truncate base_tab_def;
-insert into base_tab_def values (1);
-insert into base_tab_def values (2), (3);
-insert into base_tab_def values (4, default, default, default, default);
-insert into base_tab_def values (5, default, default, default, default),
-                                (6, default, default, default, default);
-insert into base_tab_def_view values (11);
-insert into base_tab_def_view values (12), (13);
-insert into base_tab_def_view values (14, default, default, default, default);
-insert into base_tab_def_view values (15, default, default, default, default),
-                                     (16, default, default, default, default);
-insert into base_tab_def_view values (17), (default);
-select * from base_tab_def order by a;
-
--- Using an unconditional DO INSTEAD rule should also cause NULLs to be
--- inserted where there are no view defaults.
-drop trigger base_tab_def_view_instrig on base_tab_def_view;
-drop function base_tab_def_view_instrig_func();
-create rule base_tab_def_view_ins_rule as on insert to base_tab_def_view
-  do instead insert into base_tab_def values (new.a, new.b, new.c, new.d, new.e);
-truncate base_tab_def;
-insert into base_tab_def values (1);
-insert into base_tab_def values (2), (3);
-insert into base_tab_def values (4, default, default, default, default);
-insert into base_tab_def values (5, default, default, default, default),
-                                (6, default, default, default, default);
-insert into base_tab_def_view values (11);
-insert into base_tab_def_view values (12), (13);
-insert into base_tab_def_view values (14, default, default, default, default);
-insert into base_tab_def_view values (15, default, default, default, default),
-                                     (16, default, default, default, default);
-insert into base_tab_def_view values (17), (default);
-select * from base_tab_def order by a;
-
--- A DO ALSO rule should cause each row to be inserted twice. The first
--- insert should behave the same as an auto-updatable view (using table
--- defaults, unless overridden by view defaults). The second insert should
--- behave the same as a rule-updatable view (inserting NULLs where there are
--- no view defaults).
-drop rule base_tab_def_view_ins_rule on base_tab_def_view;
-create rule base_tab_def_view_ins_rule as on insert to base_tab_def_view
-  do also insert into base_tab_def values (new.a, new.b, new.c, new.d, new.e);
-truncate base_tab_def;
-insert into base_tab_def values (1);
-insert into base_tab_def values (2), (3);
-insert into base_tab_def values (4, default, default, default, default);
-insert into base_tab_def values (5, default, default, default, default),
-                                (6, default, default, default, default);
-insert into base_tab_def_view values (11);
-insert into base_tab_def_view values (12), (13);
-insert into base_tab_def_view values (14, default, default, default, default);
-insert into base_tab_def_view values (15, default, default, default, default),
-                                     (16, default, default, default, default);
-insert into base_tab_def_view values (17), (default);
-select * from base_tab_def order by a, c NULLS LAST;
-
-drop view base_tab_def_view;
-drop table base_tab_def;
-
--- Test defaults with array assignments
-create table base_tab (a serial, b int[], c text, d text default 'Table default');
-create view base_tab_view as select c, a, b from base_tab;
-alter view base_tab_view alter column c set default 'View default';
-insert into base_tab_view (b[1], c, a)
-values (1, default, default), (10, 'C value', 100);
-select * from base_tab order by a;
-drop view base_tab_view;
-drop table base_tab;
